/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.distribution.packaging.impl.importer;

import javax.annotation.Nonnull;
import java.io.InputStream;
import java.util.Arrays;

import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.distribution.packaging.DistributionPackage;
import org.apache.sling.distribution.packaging.DistributionPackageImportException;
import org.apache.sling.distribution.packaging.DistributionPackageImporter;
import org.apache.sling.distribution.packaging.DistributionPackageInfo;
import org.apache.sling.distribution.packaging.impl.DistributionPackageUtils;
import org.apache.sling.distribution.serialization.DistributionPackageBuilder;
import org.apache.sling.distribution.serialization.DistributionPackageReadingException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * {@link org.apache.sling.distribution.packaging.DistributionPackageImporter} implementation which imports a FileVault
 * based {@link org.apache.sling.distribution.packaging.DistributionPackage} locally.
 */

public class LocalDistributionPackageImporter implements DistributionPackageImporter {

    private final Logger log = LoggerFactory.getLogger(getClass());

    private final DistributionPackageBuilder packageBuilder;

    public LocalDistributionPackageImporter(DistributionPackageBuilder packageBuilder) {

        if (packageBuilder == null) {
            throw new IllegalArgumentException("A package builder is required");
        }

        this.packageBuilder = packageBuilder;
    }


    public void importPackage(@Nonnull ResourceResolver resourceResolver, @Nonnull DistributionPackage distributionPackage) throws DistributionPackageImportException {
        try {
            boolean success = packageBuilder.installPackage(resourceResolver, distributionPackage);


            if (!success) {
                log.warn("could not install distribution package {}", distributionPackage.getId());
            }

        } catch (Exception e) {
            log.error("cannot import a package from the given stream of type {}", distributionPackage.getType());
            throw new DistributionPackageImportException(e);
        }
    }

    public DistributionPackageInfo importStream(@Nonnull ResourceResolver resourceResolver, @Nonnull InputStream stream) throws DistributionPackageImportException {
        DistributionPackage distributionPackage = null;
        try {
            distributionPackage = packageBuilder.readPackage(resourceResolver, stream);

            boolean success = packageBuilder.installPackage(resourceResolver, distributionPackage);

            if (!success) {
                log.warn("could not install distribution package {}", distributionPackage.getId());
            }

            return distributionPackage.getInfo();
        } catch (DistributionPackageReadingException e) {
            throw new DistributionPackageImportException("cannot read a package from the given stream", e);
        } finally {
            DistributionPackageUtils.deleteSafely(distributionPackage);
        }
    }

}
